#include <autoconf.h>
#include <string.h>
#include <herrno.h>
#include <scan.h>
#include <map.h>
#include <reset_cpu.h>

#define AR_GHEADER_MAGIC_NUM "!<arch>"
#define AR_GHEADER_MAGIC_NUM_SIZE 7

static void error(const char *message)
{
	hprintf("\n\nERROR: %s\n\n -- System halted\n", message);

#if defined(CONFIG_REBOOT_FATAL_ERROR)
	hprintf("reboot now\n");
	reset_cpu();	/* time to reboot */
#else
	while (1);	/* halt */
#endif
}

static void read_data(void *buf, addr_t addr, size_t size)
{
	unsigned char *p = buf;

	while (size--)
		*p++ = read8(addr++);
}

static void is_ar_formart(addr_t addr)
{
	size_t size = AR_GHEADER_MAGIC_NUM_SIZE;
	char header_magic_num[AR_GHEADER_MAGIC_NUM_SIZE + 1];

	read_data(header_magic_num, addr, size);
	header_magic_num[AR_GHEADER_MAGIC_NUM_SIZE] = '\0';

	if (strcmp(header_magic_num, AR_GHEADER_MAGIC_NUM) != 0)
		error("Not AR formart");
}

static void del_space(char *buf, size_t len)
{
	while (len--) {
		if (*buf == 0x20)
			*buf = '\0';
		buf++;
	}
}

static void analyze_file_header(addr_t addr, size_t scan_size,
				size_t *header_size, size_t *file_size)
{
	addr_t current = addr;
	unsigned char magicnum[2];
	char buf[16];

	memset(magicnum, 0x00, sizeof(magicnum));
	memset(buf, 0x00, sizeof(buf));

	while (scan_size--) {
		read_data(magicnum, current, 2);
		if (magicnum[0] == 0x60 && magicnum[1] == 0x0A) {
			read_data(buf, current - 10, 10);
			del_space(buf, strlen(buf));
			scan(buf, file_size);
			*header_size = current + 2 - addr;
			return;
		}
		current++;
	}

	error("Not AR Item");
}

void ar_search_item(addr_t start_addr, size_t size, int num, addr_t *item_addr,
		    size_t *item_size)
{
	int i;
	addr_t fiie_header_start = start_addr + AR_GHEADER_MAGIC_NUM_SIZE;
	size_t fiie_header_size = 0;
	size_t file_size = 0;

	is_ar_formart(start_addr);

	for (i = 0; i < num; i++) {
		analyze_file_header(fiie_header_start, size,
				    &fiie_header_size, &file_size);
		*item_addr = fiie_header_start + fiie_header_size;
		*item_size = file_size;
		fiie_header_start += fiie_header_size + file_size;
		size -= fiie_header_size + file_size;
	}
}
